using UnityEngine;
using System.Collections;

/*
SPGTODO:
[] PlatformPlacementInfo should probably have more member functions.
*/

public class ProceduralLevelGenerator : MonoBehaviour
{
    public int LevelLength;
    public int StartPlatformLength = 5;
    public int EndPlatformLength = 5;
    public float BuildingBlockWidth;
    public Transform PrefabBuildingBlock_FlatPlatform;
    public Transform BuildingBlocks_Parent;
    public Transform Monster;
    public Transform Monsters_Parent;
    public Transform HealthPickup;
    public Transform HealthPickup_Parent;
    public Vector3 MonsterRotation;
    public float PlatformPlacementMinY = 0.0f;
    public float PlatformPlacementMaxY = 10.0f;
    public float PlatformPlacementMaxDeltaY = 5.0f;
    public int PlatformLengthMin = 1;
    public int PlatformLengthMax = 4;
    public float OddsOfMonsterExistence = 0.25f;
    public float OddsOfHealthPickupExistence = 0.1f;
    public float HealthPickupYMin = 1.0f;
    public float HealthPickupYMax = 3.0f;


    private enum PlatformType
    {
        None,
        Flat
    }

    private class PlatformPlacementInfo
    {
        public PlatformType PlatformType;
        public float PositionY;
        public bool HasMonster;
        public bool HasHealthPickup;

        public PlatformPlacementInfo(PlatformType pt, float posY, bool hasMonster, bool hasHealthPickup)
        {
            PlatformType = pt;
            PositionY = posY;
            HasMonster = hasMonster;
            HasHealthPickup = hasHealthPickup;
        }
    }

    void CreatePlatformsFromPlacementInfo(PlatformPlacementInfo[] platformInfo)
    {
        for (int i = 0; i < platformInfo.GetLength(0); ++i)
        {
            PlatformPlacementInfo placementInfo = platformInfo[i];

            if (placementInfo.PlatformType == PlatformType.None)
            {
                continue;
            }

            Vector3 placementPosition = new Vector3(BuildingBlockWidth * i, placementInfo.PositionY, 0.0f);
            Transform createBlock = (Transform)Instantiate(PrefabBuildingBlock_FlatPlatform, placementPosition, Quaternion.identity);
            createBlock.parent = BuildingBlocks_Parent;

            if (placementInfo.HasMonster)
            {
                placementPosition = new Vector3(BuildingBlockWidth * i, placementInfo.PositionY + 0.1f, 0.0f);
                Transform createMonster = (Transform)Instantiate(Monster, placementPosition, Quaternion.Euler(MonsterRotation));
                createMonster.parent = Monsters_Parent;
            }

            if (placementInfo.HasHealthPickup)
            {
                placementPosition = new Vector3(BuildingBlockWidth * i, placementInfo.PositionY + Random.Range(HealthPickupYMin, HealthPickupYMax), 0.0f);
                Transform createHealth = (Transform)Instantiate(HealthPickup, placementPosition, Quaternion.Euler(MonsterRotation));
                createHealth.parent = HealthPickup_Parent;                
            }
        }
    }

    float FindPreviousYPosition(PlatformPlacementInfo[] platformInfo, int startIndex)
    {
        while (startIndex >= 0)
        {
            if (platformInfo[startIndex].PlatformType != PlatformType.None)
            {
                return platformInfo[startIndex].PositionY;
            }
            startIndex--;
        }

        return 0.0f;
    }

    void FillOutPlacementInfo(ref PlatformPlacementInfo[] platformInfo)
    {
        int currentPlatformInfoIndex = 0;

        for (int i = 0; i < StartPlatformLength; ++i)
        {
            platformInfo[currentPlatformInfoIndex].PlatformType = PlatformType.Flat;
            platformInfo[currentPlatformInfoIndex].PositionY = 0.0f;
            currentPlatformInfoIndex++;
        }

                
        while (currentPlatformInfoIndex < LevelLength - EndPlatformLength)
        {
            if (platformInfo[currentPlatformInfoIndex - 1].PlatformType != PlatformType.None)
            {
                currentPlatformInfoIndex++;
                continue;
            }

            float platformPositionY = Random.Range(PlatformPlacementMinY, PlatformPlacementMaxY);
            float previousPositionY = FindPreviousYPosition(platformInfo, currentPlatformInfoIndex);
            if (platformPositionY > (previousPositionY + PlatformPlacementMaxDeltaY))
            {
                platformPositionY = previousPositionY + PlatformPlacementMaxDeltaY;
            }

            int platformLength = Random.Range(PlatformLengthMin, PlatformLengthMax);
            for (int i = 0; i < platformLength; ++i)
            {
                bool hasMonster = (Random.Range(0.0f, 1.0f) < OddsOfMonsterExistence);
                bool hasHealthPickup = (Random.Range(0.0f, 1.0f) < OddsOfHealthPickupExistence);

                platformInfo[currentPlatformInfoIndex].PlatformType = PlatformType.Flat;
                platformInfo[currentPlatformInfoIndex].PositionY = platformPositionY;
                platformInfo[currentPlatformInfoIndex].HasMonster = hasMonster;
                platformInfo[currentPlatformInfoIndex].HasHealthPickup = hasHealthPickup;
                currentPlatformInfoIndex++;

                if (currentPlatformInfoIndex > (LevelLength - EndPlatformLength))
                {
                    currentPlatformInfoIndex = LevelLength - EndPlatformLength;
                    break;
                }
            }            
        }

        for (int i = 0; i < EndPlatformLength; ++i)
        {
            platformInfo[currentPlatformInfoIndex].PlatformType = PlatformType.Flat;
            platformInfo[currentPlatformInfoIndex].PositionY = 0.0f;
            currentPlatformInfoIndex++;
        }
    }

    void GenerateLevel()
    {
        PlatformPlacementInfo[] platformInfo = new PlatformPlacementInfo[LevelLength];
        for (int i = 0; i < LevelLength; ++i)
        {
            platformInfo[i] = new PlatformPlacementInfo(PlatformType.None, -1.0f, false, false);
        }

        FillOutPlacementInfo(ref platformInfo);
        CreatePlatformsFromPlacementInfo(platformInfo);
    }

	void Start()
    {
        GenerateLevel();
	}
	
	void Update()
    {	
	}
}
