using UnityEngine;
using System.Collections;

public class ProgressController : MonoBehaviour
{
    public ProceduralLevelGenerator LevelGenerator;
    public Transform Hero;
    public float KillPositionY = -7.0f;
    public int StartingHealth = 3;
    public float DistanceBeforeSpeedIncrement = 100.0f;
    public float SpeedIncrementAmount = 0.1f;

    private PlatformerController heroPlatformerController;
    private int currentHealth;
    private int distanceFromStart;
    private bool isDead;
    private bool hasWon;
    private float currentOverallSpeedMultiplier = 1.0f;
    private float timeStarted;

	void Start()
	{
        heroPlatformerController = (PlatformerController)Hero.gameObject.GetComponent("PlatformerController"); 
	    currentHealth = StartingHealth;
	    distanceFromStart = 0;
	    timeStarted = Time.time;
	    isDead = false;
	    hasWon = false;
	}

    void KillHero()
    {
        isDead = true;
        Destroy(Hero.gameObject);
    }

    void LevelWin()
    {
        hasWon = true;
        Destroy(Hero.gameObject);
    }

    public void TakeDamage()
    {
        currentHealth--;
        if (currentHealth <= 0)
        {
            KillHero();
        }
    }

    public void Heal()
    {
        currentHealth++;
    }

    void Update()
    {
        if (isDead || Hero == null)
        {
            return;
        }        

        Vector3 levelStartPositionNoYorZ = new Vector3(LevelGenerator.transform.position.x, 0.0f, 0.0f);
        Vector3 ourPositionNoYorZ = new Vector3(Hero.transform.position.x, 0.0f, 0.0f);
        distanceFromStart = Mathf.RoundToInt(Mathf.Abs((ourPositionNoYorZ - levelStartPositionNoYorZ).magnitude));

        if (distanceFromStart > (LevelGenerator.LevelLength - 1) * LevelGenerator.BuildingBlockWidth && Time.time > timeStarted + 3.0f)
        {
            LevelWin();
        }

	    currentOverallSpeedMultiplier = 1.0f + Mathf.FloorToInt(distanceFromStart / DistanceBeforeSpeedIncrement) * SpeedIncrementAmount;

        heroPlatformerController.setOverallSpeedMultiplier(currentOverallSpeedMultiplier);

        if (Hero.position.y < KillPositionY)
        {
            KillHero();
            return;
        }
	}

    void OnGUI()
    {
        GUI.Box(new Rect(0.0f, Screen.height - 25.0f, 125.0f, 25.0f), "Health: " + currentHealth);
        GUI.Box(new Rect(125.0f, Screen.height - 25.0f, 125.0f, 25.0f), "Distance: " + distanceFromStart);
        GUI.Box(new Rect(Screen.width - 250.0f, Screen.height - 25.0f, 250.0f, 25.0f), "Spacebar = Jump  Enter = Shoot");

        if (hasWon)
        {            
            GUI.Box(new Rect(Screen.width / 2 - 200, Screen.height / 2 - 100, 400.0f, 175.0f), "You did it! Good job dude.");
            if (GUI.Button(new Rect(Screen.width / 2 - 100, Screen.height / 2 - 50, 200, 40), "Restart the Game"))
            {
                Application.LoadLevel(0);
            }
            if (GUI.Button(new Rect(Screen.width / 2 - 100, Screen.height / 2, 200, 40), "Quit"))
            {
                Application.Quit();
            }
        }
        else if (isDead)
        {
            GUI.Box(new Rect(Screen.width / 2 - 200, Screen.height / 2 - 100, 400.0f, 175.0f), "It's over man, but you made it this far: " + distanceFromStart);
            if (GUI.Button(new Rect(Screen.width / 2 - 100, Screen.height / 2 - 50, 200, 40), "Restart the Game"))
            {
                Application.LoadLevel(0);
            }
            if (GUI.Button(new Rect(Screen.width / 2 - 100, Screen.height / 2, 200, 40), "Quit"))
            {
                Application.Quit();
            }
        }        
    }
}
